<?php
/*
AccessKeys CLASS (PHP 4)
========================

Author: Gez Lemon (http://juicystudio.com)
Usage:
$objAK = new AccessKeys;

The constructor for the AccessKey class takes an optional array to help set up the 
class. The array may contain names and values, or names and an array that includes
the value and a suggested access key, or a mixture of the two. The value may either 
be a URI, or an idref used with the label element. For example, the 
following defines an array of name value pairs (search contains an idref, intended to
be used with a label):

$arBindings = array("Home"=>"/index.php",
					"Skip Navigation"=>"#maincontent",
					"Search"=>"search",
					"Accessibility Statement"=>"/access.php");

$objAK = new AccessKeys($arBindings);

The next example provides the value as an array that includes a suggested accesskey 
value.

$arBindings = array("Home"=>array("/index.php", "1"),
					"Skip Navigation"=>array("#maincontent", "2"),
					"Search"=>array("search", "4"),
					"Accessibility Statement"=>array("/access.php", "0"));

$objAK = new AccessKeys($arBindings);

The final example uses a mixture of name/value pairs, and a name with arrays that 
contain the suggested access key.

$arBindings = array("Home"=>array("/index.php", "1"),
					"Skip Navigation"=>array("#maincontent", "2"),
					"Search"=>array("search", "4"),
					"Quality Assurance"=>"/services.php",
					"Accessibility Statement"=>array("/access.php", "0"));

$objAK = new AccessKeys($arBindings);


AccessKey Methods
==================

The following lists the methods that can be used with the AccessKeys Class:


addAccessitem Method
====================

The addAccessitem method allows a single item to be added to the collection 
of items that may be assigned an accesskey.

@parameter String $strItem (The name of the item to be added to the collection)
@parameter String $strValue (The value - may also be an array)

Examples:
$objAK->addAccessitem("Meat", "meat.php");
$objAK->addAccessitem("Fish", array("fish.php", "9"));


removeAccessItem Method
=======================

The removeAccessitem method removes an item from the collection of items that 
may be assigned an access key.

@parameter String $strItem (The name of the item to be removed from the collection)

Example:
$objAK->removeAccessitem("Meat");


isValid Method
==============

The isValid method returns a Boolean value indicating whether or not the values 
assigned were valid.

@parameter none

Example:
if (!$objAK->isValid())
	echo $objAK->getErrors();


getErrors Method
================

The getErrors method returns a string containing errors from form submissions, 
or an empty string if there are no errors. The getErrors method should be used 
with the isValid method.

@parameter none

Example:
if (!$objAK->isValid())
	echo $objAK->getErrors();


setCookies Method
=================

The setCookies method returns a Boolean value indicating whether or not the
values were valid accesskey values. If the values are inappropriate, isValid
returns false, and getErrors can be used to retrieve the errors. Cookies must
be set in the head of the document, and the current document must be reloaded
before the values are effective. The return value from the setCookies method 
can be used to determine whether or not to reload the current page.

@parameter none

Example:

if ($objAK->setCookies())
	header("Location: $PHP_SELF");


removeAllBindings Method
========================

The removeAllBindings method removes all access keys defined for the collection.

@parameter none

Example:

$objAK->removeAllBindings();


displayOptions Method
=====================

The displayOptions method displays a definition list where the definition terms 
are the names in the collection, and the definitions contain the assigned key, 
and suggested key if available.

@parameter String $strID (optional parameter to assign an id attribute to the definition list)

Examples:
$objAK->displayOptions();
$objAK->displayOptions("someid");


generateForm Method
===================

The generateForm method generates form sections based on the names provided 
in the collection. The method takes an optional value to indicate whether the 
generated markup is XHTML (by default, the value is false).

@parameter Boolean $bXHTML (optional parameter to determine markup format)

Examples:

The following generates the form markup for HTML:

$objAK->generateForm();

The next example generates the form markup for XHTML:

$objAK->generateForm(true);


useDefaults Method
==================

The useDefault method assigns all of the values that have been suggested as 
default keys by the author, if any have been provided.

@parameter none

Example:

$objAK->useDefaults();
header("Location: $PHP_SELF");


getAccessitem Method
====================

The getAccessitem method returns the markup for a particular item, including 
the accesskey if one has been defined. If the item doesn't exist in the collection, 
the method returns an empty string. The accesskey can be ignored using the 
$bKey parameter, in which case, just the markup will be returned. As access keys
may be assigned to links, labels, or form controls, an optional parameter allows
the markup to return label tags. By default, the markup returns anchor tags.

@parameter String $strItem (The item to be returned)
@parameter Boolean $bKey (Indicates whether the accesskey value should be included)
@parameter String $strElement (Element to be returned. Default: "link", optional "label")

Examples:

An example of how this might be used in a navigation list

<ul id="navigation">
<li><?php echo $objAK->getAccessitem("Home", true); ?></li>
<li><?php echo $objAK->getAccessitem("Contact Us", true); ?></li>
<li><?php echo $objAK->getAccessitem("Quality Assurance", false); ?></li>
</ul>

An example of how this might be used with a form element
$objAK->getAccessitem("Search", true, "label"); 
<input type="text" id="search" name="search" size="10">

*/
class AccessKeys
{
	// Data
	var $dtExpires;
	var $bValid;
	var $strError;
	var $arBindings = array();

	// Constructor
	function AccessKeys($arBindings = array())
	{
		if (!empty($arBindings))
			foreach($arBindings as $strName => $strValue)
				$this->arBindings[$strName] = $strValue;

		$this->dtExpires = time() + 60 * 60 * 24 * 365;
		$this->bValid = true;
	}

	// Methods
	function addAccessitem($strItem, $strValue)
	{
		$this->arBindings[$strItem] = $strValue;
	}
	
	function removeAccessitem($strItem)
	{
		unset($this->arBindings[$strItem]);
	}

	function isValid()
	{
		return $this->bValid;
	}

	function getErrors()
	{
		return $this->strError;
	}

	function setCookies()
	{
		$arKeys = array();
		$this->bValid = true;
		$this->strError = "<div id=\"jserror\">\n";
		$this->strError .= "<h2>Submission Error</h2>\n";
		$this->strError .= "<p>Please review the following</p>\n";
		$this->strError .= "<ul>\n";
		
		$iCounter = 0;
		foreach($this->arBindings as $strName => $strValue)
		{
			$strFind = strtolower(preg_replace("/\s*/", "", $strName));
			$strAccess = trim($_POST[$strFind]);
			$strKey = trim($_POST[$strFind. "binding"]);
			if ($strAccess)
			{
				if (!ctype_alnum($strKey) || strlen($strKey) == 0 || strlen($strKey) > 1)
				{
					$this->bValid = false;
					$this->strError .= "\t<li>Please provide a single alphanumeric character for $strName</li>\n";
				}
				$arKeys[$iCounter++] = $strKey;
			}
		}

		for ($iInner=0; $iInner<count($arKeys)-1; $iInner++)
			for ($iOuter=$iInner+1; $iOuter<count($arKeys); $iOuter++)
				if ($arKeys[$iInner] == $arKeys[$iOuter] && strlen($arKeys[$iInner]) > 0)
				{
					$this->bValid = false;
					$this->strError .= "\t<li>Please ensure the access key is unique. The value {$arKeys[$iInner]} was used more than once</li>\n";
				}

		if (!$this->bValid)
		{
			$this->strError .= "</ul>\n";
			$this->strError .= "</div>\n";
		}
		else
		{
			$this->strError = "";
			foreach($this->arBindings as $strName => $strValue)
			{
				$strName = strtolower(preg_replace("/\s*/", "", $strName));
				$strAccess = $_POST[$strName];
				$strKey = $_POST[$strName. "binding"];
				if ($strAccess)
					setcookie($strName, $strKey, $this->dtExpires, "/");
				else
					setcookie($strName, "", $this->dtExpires, "/");
			}
		}
		
		return $this->bValid;
	}
	
	function removeAllBindings()
	{
		foreach($this->arBindings as $strName => $strValue)
			setcookie(strtolower(preg_replace("/\s*/", "", $strName)), "", $this->dtExpires, "/");
	}

	function displayOptions($strID = "")
	{
		if (is_array($this->arBindings) && !empty($this->arBindings))
		{
			if (strlen($strID) > 0)
				echo "<dl id=\"$strID\">\n";
			else
				echo "<dl>\n";
			foreach ($this->arBindings as $strName => $strValue)
			{
				$strKey = $_COOKIE[strtolower(preg_replace("/\s*/", "", $strName))];
				echo "\t<dt>$strName</dt>\n";
				if ($strKey)
					echo "\t<dd>Access key: <kbd>$strKey</kbd>.</dd>\n";
				else
					echo "\t<dd>No access key defined.</dd>\n";
				if (is_array($strValue) && !empty($strValue))
					echo "\t<dd>Suggested key: <kbd>{$strValue[1]}</kbd></dd>\n";
			}
			echo "</dl>\n";
		}
	}
	
	function generateForm($bXHTML = false)
	{
		if (is_array($this->arBindings) && !empty($this->arBindings))
		{
			foreach ($this->arBindings as $strName => $strValue)
			{
				$strID = strtolower(preg_replace("/\s*/", "", $strName));
				echo "<fieldset>\n";
				echo "<legend>Access Key Settings for $strName</legend>\n";
				echo "<p>\n";
				echo "\t<input type=\"checkbox\" name=\"$strID\" id=\"jsak_$strID\" value=\"$strID\"";
				if (strlen($_COOKIE[$strID]) > 0)
					echo " checked=\"checked\"";
				if ($bXHTML)
					echo "/>\n";
				else
					echo ">\n";
				echo "\t<label for=\"jsak_$strID\">$strName</label>\n";
				echo "</p>\n";
				echo "<p>\n";
				echo "\t<label for=\"jsak_{$strID}binding\">Binding for $strName Key</label>\n";
				echo "\t<input type=\"text\" size=\"4\" value=\"{$_COOKIE[$strID]}\" id=\"jsak_{$strID}binding\" name=\"{$strID}binding\"";
				if ($bXHTML)
					echo "/>\n";
				else
					echo ">\n";
				echo "</p>\n";
				echo "</fieldset>\n";
			}
		}
	}
	
	function useDefaults()
	{
		foreach ($this->arBindings as $strName => $strValue)
			if (is_array($strValue) && !empty($strValue))
			{
				$strName = strtolower(preg_replace("/\s*/", "", $strName));
				setcookie($strName, $strValue[1], $this->dtExpires, "/");
			}
	}

	function getAccessitem($strItem, $bKey, $strElement="link")
	{
		if (!empty($this->arBindings[$strItem]))
		{
			$strKey = $_COOKIE[strtolower(preg_replace("/\s*/", "", $strItem))];
			if (is_array($this->arBindings[$strItem]) && !empty($this->arBindings[$strItem]))
				$strValue = $this->arBindings[$strItem][0];
			else
				$strValue = $this->arBindings[$strItem];

			if ($bKey && $strKey)
				if ($strElement == "label")
					return "<label for=\"$strValue\" accesskey=\"$strKey\">$strItem</label>";
				else
					return "<a href=\"$strValue\" accesskey=\"$strKey\">$strItem</a>";
			else
				if ($strElement == "label")
					return "<label for=\"$strValue\">$strItem</label>";
				else
					return "<a href=\"$strValue\">$strItem</a>";
		}
		else
			return "";
	}
}
